from .collection import (
    Collection,
    CollectionItemBase,
    FileItem,
    ImageCollection,
    ImageItem,
    SVGItem,
)
from .fields import CollectionField, CollectionItem, FileField, ImageField
from .file import UploadedFile
from .image import UploadedImage

__all__ = [
    "UploadedFile",
    "UploadedImage",
    "FileField",
    "ImageField",
    "CollectionField",
    "CollectionItemBase",
    "CollectionItem",
    "Collection",
    "ImageCollection",
    "FileItem",
    "SVGItem",
    "ImageItem",
]



from django import forms
from django.urls import reverse_lazy

from .base import FileWidgetBase
from .mixins import FileUploaderWidgetMixin


class ImageWidget(FileUploaderWidgetMixin, FileWidgetBase):
    template_name = "paper_uploads/image_widget.html"

    @property
    def media(self):
        return forms.Media(
            js=[
                "paper_uploads/dist/widget.js",
            ],
            css={
                "screen": [
                    "paper_uploads/dist/widget.css",
                ],
            },
        )

    def get_context(self, name, value, attrs):
        context = super().get_context(name, value, attrs)

        # urls
        info = self.model._meta.app_label, self.model._meta.model_name
        context.update(
            {
                "upload_url": reverse_lazy("admin:%s_%s_upload" % info),
                "change_url": reverse_lazy("admin:%s_%s_change" % info),
                "delete_url": reverse_lazy("admin:%s_%s_delete" % info),
            }
        )
        return context



from typing import Any, Dict

from django.db import models
from django.db.models.fields.files import FieldFile
from django.utils.translation import gettext_lazy as _

from ..conf import settings
from ..storage import upload_storage
from ..utils import filesizeformat
from .base import FileFieldResource


class UploadedFile(FileFieldResource):
    file = models.FileField(
        _("file"),
        max_length=255,
        storage=upload_storage,
        upload_to=settings.FILES_UPLOAD_TO,
    )
    display_name = models.CharField(_("display name"), max_length=255, blank=True)

    class Meta(FileFieldResource.Meta):
        verbose_name = _("file")
        verbose_name_plural = _("files")

    def save(self, *args, **kwargs):
        if not self.pk and not self.display_name:
            self.display_name = self.basename
        super().save(*args, **kwargs)

    def get_file(self) -> FieldFile:
        return self.file

    def set_file(self, value):
        self.file = value

    def get_file_field(self) -> models.FileField:
        return self._meta.get_field("file")

    def as_dict(self) -> Dict[str, Any]:
        return {
            **super().as_dict(),
            "name": self.display_name,
            "file_info": "({ext}, {size})".format(
                ext=self.extension, size=filesizeformat(self.size)
            ),
        }
